from .base_component import BaseComponent


class Image(BaseComponent):
    name: str = "image"

    def __init__(self, src: str, alt: str = ""):
        self.props['src'] = src
        self.props['alt'] = alt

    def validate_props(self) -> str:
        if not self.props['src']:
            return "src is required"
        return ''



from .base_component import BaseComponent


class Link(BaseComponent):
    name: str = "link"

    def __init__(self, text: str, url: str):
        self.props['text'] = text
        self.props['url'] = url

    def validate_props(self) -> str:
        if not self.props['text']:
            return "text is required"
        if not self.props['url']:
            return "url is required"

    def to_message(self) -> str:
        err = self.validate_props()
        if err:
            raise err

        return f"[{self.props['text']}]({self.props['url']})"



from .base_component import BaseComponent


class Row(BaseComponent):
    name: str = "row"

    def __init__(self, children: list[BaseComponent]):
        self.props['children'] = children

    def validate_props(self) -> str:
        if not self.props['children']:
            return "children is required"
        return ''



import contextvars
import logging
import os

NodeClsKey = "x-apaas-node-cls"

CtxVar = contextvars.ContextVar(NodeClsKey)

try:
    from _sys import CtxVar  # type: ignore
except Exception as e:
    logging.info(f'fail to load CtxVar {e}')


def ctx():
    return contextvars.copy_context()


def get_ctx():
    try:
        return CtxVar.get()
    except LookupError:
        CtxVar.set({})
        return CtxVar.get()


def get_ctx_key(key):
    try:
        return get_ctx()[key]
    except Exception as err:
        logging.info(f'key `{key}` not found in context {err}')
        return None


def get_from_credential(key: str):
    credential = get_ctx_key("credential")
    if credential is None:
        return ""
    return credential[key]


def is_local_dev():
    return os.environ.get("AILY_SDK_LOCAL_DEBUG") == "true"


def get_client_id():
    if is_local_dev():
        return os.environ.get("AILY_SDK_CLIENT_ID")
    return get_from_credential("clientID")


def get_client_secret():
    if is_local_dev():
        return os.environ.get("AILY_SDK_CLIENT_SECRET")
    return get_from_credential("clientSecret")


def get_domain():
    if is_local_dev():
        return os.environ.get("AILY_SDK_DOMAIN")
    return get_from_credential("domain")


def get_by_headers(key):
    headers = get_ctx_key("sysHeaders")
    if headers is None:
        return ""
    return headers.get(key)



from datetime import datetime
from urllib.parse import urljoin
from ..common.context import get_client_id, get_client_secret, get_domain
from ..common.util import get_http_session

AppTokenRefreshTime = 20 * 60 * 1000


class AppCredential:
    def __init__(self):
        self.expire_time = 0
        self.tenant_info = None
        self.id = get_client_id()
        self.secret = get_client_secret()

    def get_token(self):
        remain_time = self.expire_time - datetime.now().timestamp()
        if remain_time > AppTokenRefreshTime and self.token:
            return self.token
        self.fetch_token()
        return self.token

    def get_tenant_info(self):
        if not self or not self.id or not self.secret:
            self.id = get_client_id()
            self.secret = get_client_secret()
        if self.tenant_info:
            return self.tenant_info
        self.fetch_token()
        return self.tenant_info

    def fetch_token(self):
        domain = get_domain()
        if not domain:
            raise Exception("get domain failed")

        s = get_http_session()
        res = s.post(urljoin(domain, "/auth/v1/appToken"), json={
            "clientId": self.id,
            "clientSecret": self.secret,
            "withTenantInfo": True,
        })
        data = res.json()

        if not data['data']:
            raise Exception("get token failed")

        data = data['data']
        self.token = data["accessToken"]
        self.namespace = data["namespace"]
        self.expire_time = data["expireTime"]
        self.tenant_info = data["tenantInfo"]



from typing import Literal
from .base_component import BaseComponent

TitleStyle = Literal[
    "blue",
    "wathet",
    "turquoise",
    "green",
    "yellow",
    "orange",
    "red",
    "carmine",
    "violet",
    "purple",
    "grey",
    "default",
]


class Title(BaseComponent):
    name: str = "title"

    def __init__(self, style: TitleStyle, children: str):
        self.props['style'] = style
        self.props['children'] = children

    def validate_props(self) -> str:
        if not self.props['children']:
            return "children is required"
        return ''



from typing import Any
from urllib.parse import urljoin
import json
from ..common.context import get_domain, get_by_headers

from ..credential import AppCredential
from ..common.util import get_http_session

defaultOptions = {
    "timeout": 60 * 1000 * 5,
    "retry_times": 0,
}


def call_action(
        action_api_name: str,
        action_data: dict,
        options=None,
        is_async=False,
) -> Any:
    if options is None:
        options = {}
    credential = AppCredential()
    token = credential.get_token()
    domain = get_domain()

    if not token:
        raise Exception('get token failed')

    http = get_http_session(token)
    current_skill_id = get_by_headers('x-aily-skill-instance-id')
    if current_skill_id is None and action_api_name == 'action:brn:cn:spring:all:all:connector_action_runtime' \
                                                       ':/spring_sdk_send_message':
        raise Exception('发送消息接口暂不支持直接调试, 需要点击技能调试.')

    action_data.update({'skillInstanceID': current_skill_id})
    req = {
        'customerBizId': 'spring',
        'data': {
            'actionApiName': action_api_name,
            'actionData': json.dumps(action_data),
            'options': json.dumps({
                **defaultOptions,
                **options,
            }),
        },
        "isAsync": is_async
    }

    path = f'/ai/v1/connector_action/namespaces/{credential.namespace}/execute_action'  # noqa: E501
    url = urljoin(domain, path)
    res = http.post(url, json=req)

    if res.status_code != 200:
        raise Exception(f'call action failed: {res.text}')

    logid = res.headers.get('x-tt-logid')
    data = res.json()
    if data['code'] != '0':
        raise Exception(f'[{logid}] call action failed: {data}')

    if data['data']['code'] not in ['null', '200', '']:
        raise Exception(f'[{logid}] parse action response error: data is empty, {data}')

    if not data['data']['data']:
        raise Exception(f'[{logid}] parse action response error: data is empty, {data}')

    return json.loads(data['data']['data'])



from typing import Union
from .base_component import BaseComponent


class Highlight(BaseComponent):
    children_wrapper: str = '\n'
    name: str = "highlight"

    def __init__(self, children: Union[str, list[str]]):
        self.props['children'] = children

    def validate_props(self) -> str:
        if not self.props['children']:
            return "children is required"
        return ''



from .base_component import BaseComponent


class OrderedList(BaseComponent):
    def __init__(self, items: list[str]):
        self.props['items'] = items

    def validate_props(self) -> str:
        if not self.props['items']:
            return "items is required"
        return ''

    def to_message(self) -> str:
        err = self.validate_props()
        if err:
            raise err

        return "\n".join(f"{idx + 1}. {item}" for idx, item in enumerate(self.props['items']))



import requests
import json
from .context import get_by_headers


def get_http_session(token: str = '') -> requests.Session:
    s = requests.Session()
    s.headers.update({
        "Authorization": token,
        "biz_id": "spring",
        "Content-Type": "application/json",
        "Rpc-Persist-AUTH-TYPE": "user",
    })

    tt_env = get_by_headers("x-tt-env")
    if tt_env:
        s.headers.update({'x-tt-env': tt_env})

    user = get_by_headers("x-kunlun-initiator")
    if user:
        user_id = json.loads(user)['_id']
        s.headers.update({'User': str(user_id)})

    return s



from .base_component import BaseComponent


class Mention(BaseComponent):
    name: str = "at"

    def __init__(self, id: str = None, email: str = None):
        self.props['id'] = id
        self.props['email'] = email

    def validate_props(self) -> str:
        if not self.props['id'] and not self.props['email']:
            return "id or email is required"



from typing import List, Mapping, Union

from .components.base_component import BaseComponent

LineType = Union[BaseComponent, str]


class MessageBuilderOptions:
    def __init__(self, line_divider: str = "\n"):
        self.line_divider = line_divider


class MessageBuilder:
    def __init__(self, options: Mapping[str, str] = {}):
        self.lines: List[LineType] = []
        self.options = MessageBuilderOptions(**options)

    def add_line(self, component: LineType):
        self.lines.append(component)
        return self

    def to_message(self) -> str:
        return self.options.line_divider.join(
            line.to_message()
            if isinstance(line, BaseComponent) else str(line)
            for line in self.lines
        )



from typing import Any, Dict, List, Literal, Optional
from .base_component import BaseComponent


RecordFieldType = Literal[
    "string",
    "number",
    "boolean",
    "array",
    "object",
    "date",
    "options",
    "persons",
    "markdown",
]


class RecordField(Dict[str, Any]):
    type: RecordFieldType
    name: str
    displayName: str = ""

    def __init__(self, type: str, name: str, displayName: Optional[str]):
        self.type = type
        self.name = name
        self.displayName = displayName


class DataRecord(BaseComponent):
    name: str = "record"

    def __init__(self, fields: List[RecordField], data: Dict[str, Any]):
        self.props['fields'] = fields
        self.props['data'] = data

    def validate_props(self) -> str:
        if not self.props['fields']:
            return "fields is required"
        if not self.props['data']:
            return "data is required"
        return ''



from .base_component import BaseComponent


class ColorText(BaseComponent):
    name: str = "font"

    def __init__(self, children: str, color: str):
        self.props['children'] = children
        self.props['color'] = color

    def validate_props(self) -> str:
        if not self.props['children']:
            return "children is required"
        if not self.props['color']:
            return "color is required"
        return ''



from .base_component import BaseComponent


class ItemList(BaseComponent):

    def __init__(self, items: list[str]):
        self.props['items'] = items

    def to_message(self) -> str:
        if not self.props['items']:
            raise "image, validate err: items is required"

        return "\n".join(f"- {item}" for item in self.props['items'])



from .base_component import BaseComponent


class Code(BaseComponent):
    def __init__(self, children: str) -> None:
        self.props['children'] = children

    def validate_props(self) -> str:
        if not self.props['children']:
            return "children is required"

    def to_message(self) -> str:
        return f"```{self.props['children']}```"



from typing import Literal, Optional, Union
from .base_component import BaseComponent

ButtonType = Literal[
    "default",
    "primary",
    "danger",
    "text",
    "primaryFilled",
    "dangerFilled",
    "primaryText",
    "dangerText"
]

ButtonSize = Literal["tiny", "small", "medium", "large"]

ButtonUrlMode = Literal["default", "raw"]


class Button(BaseComponent):
    name = "button"

    def validate_props(self) -> str:
        if not self.props['action']:
            return "action is required"
        if self.props['action'] == 'navigate' and not self.props['url']:
            return "url is required when action is navigate"
        return ''

    def __init__(self,
                 children: str,
                 action: Optional[Literal["navigate", "message"]],
                 url: str = None,
                 type: Optional[ButtonType] = None,
                 width: Optional[Union[Literal["default",
                                               "fill"], str]] = None,
                 size: Optional[ButtonSize] = None,
                 urlMode: Optional[ButtonUrlMode] = None,
                 message: Optional[str] = None,
                 skill: Optional[str] = None,
                 ):
        self.props['children'] = children
        self.props['url'] = url
        self.props['action'] = action
        self.props['type'] = type
        self.props['width'] = width
        self.props['size'] = size
        self.props['urlMode'] = urlMode
        self.props['message'] = message
        self.props['skill'] = skill



from typing import Optional, Dict
from .base_component import BaseComponent


class Chart(BaseComponent):
    name: str = "chart"

    def __init__(self, chartSpec: Dict, options: Optional[Dict] = None):
        self.props['chartSpec'] = chartSpec
        self.props['options'] = options



from typing import Union
from .base_component import BaseComponent


class Col(BaseComponent):
    name: str = "col"

    def __init__(self, flex: int, children: Union[BaseComponent, str]):
        self.props['flex'] = flex
        self.props['children'] = children



from typing import Literal
from .base_component import BaseComponent

TableColumnType = Literal[
    "string",
    "number",
    "boolean",
    "array",
    "object",
    "date",
    "options",
    "persons",
    "markdown",
]


class TableColumn():
    type: TableColumnType
    name: str
    displayName: str
    width: str

    def __init__(self, type: TableColumnType, name: str, displayName: str, width: str):
        self.type = type
        self.name = name
        self.displayName = displayName
        self.width = width


class Table(BaseComponent):
    name: str = "table"

    def __init__(self, columns: list[TableColumn], data: list[dict], pageSize: int, rowHeight: str):
        self.props['columns'] = columns
        self.props['data'] = data
        self.props['pageSize'] = pageSize
        self.props['rowHeight'] = rowHeight

    def validate_props(self) -> str:
        if not self.props['columns']:
            return "columns is required"
        if not self.props['data']:
            return "data is required"
        return ''



from .button import Button
from .image import Image
from .chart import Chart
from .table import Table
from .code import Code
from .row import Row
from .col import Col
from .color_text import ColorText
from .data_record import DataRecord
from .title import Title
from .highlight import Highlight
from .item_list import ItemList
from .link import Link
from .mention import Mention
from .note import Note
from .ordered_list import OrderedList


__all__ = [
    "Button",
    "Image",
    "Chart",
    "Table",
    "Code",
    "Row",
    "Col",
    "ColorText",
    "DataRecord",
    "Title",
    "Highlight",
    "ItemList",
    "Link",
    "Mention",
    "Note",
    "OrderedList",
]



from ..credential import AppCredential


def get_env_info():
    credential = AppCredential()
    tenant_info = credential.get_tenant_info()

    if not tenant_info:
        raise Exception("get tenant info failed")

    return {
        "appId": credential.namespace,
        "tenant": {
            "id": tenant_info['id'],
            "name": tenant_info['tenantName'],
            "type": tenant_info['tenantType'],
        },
    }



from typing import List
import json


class BaseComponent:

    children_wrapper: str = ''

    @property
    def props(self):
        if not hasattr(self, '_props'):
            self._props = {}
        return self._props

    def validate_props(self) -> str:
        return ''

    def prop_lines(self) -> List[str]:
        lines = []
        for key, value in self.props.items():
            if key == 'children' or value is None:
                continue
            if isinstance(value, str):
                lines.append(f"{key}='{value}'")
            else:
                lines.append(f"{key}={json.dumps(value, indent=2)}")
        return lines

    def prop_lines_string(self) -> str:
        lines = self.prop_lines()
        if len(lines) == 0:
            return ""
        return f" {' '.join(lines)}"

    def to_message(self) -> str:
        if not self.name:
            return ""

        err = self.validate_props()
        if err:
            raise Exception(err)

        if "children" in self.props:
            children = self.props["children"]
            children_message = ""
            if isinstance(children, list):
                children_message = "\n".join(
                    child.to_message()
                    if isinstance(child, BaseComponent) else str(child)
                    for child in children
                )
            elif isinstance(children, BaseComponent):
                children_message = children.to_message()
            else:
                children_message = str(children)

            return "".join([
                f"<{self.name}{self.prop_lines_string()}>",
                self.children_wrapper, children_message, self.children_wrapper,
                f"</{self.name}>",
            ])
        else:
            return f"<{self.name}{self.prop_lines_string()} />"
